;*** Visinski radar s PIC 16F876/16F876A - S53MV - 27.09.2007 ***

;*** Uporaba portov - takt 20MHz ***
;analogni vhod RA0 - detektor 28kHz
;analogni vhod RA1 - detektor 32kHz
;digitalni izhod RA2 - perioda zage bit 0
;digitalni izhod RA3 - perioda zage bit 1
;digitalni izhod RA4 - perioda zage bit 2
;digitalni izhod RA5 - perioda zage bit 3
;digitalni vhod RB0 - tipka na maso
;digitalni izhod RB1 - perioda zage bit 4 
;digitalni izhod RB2 - LCD /CS1
;digitalni izhod RB3 - LCD /CS2
;digitalni izhod RB4 - RS-232 podatki za govor
;digitalni izhod RB5 - LCD R/W
;digitalni izhod RB6 - LCD D/I
;digitalni izhod RB7 - LCD E
;digitalni vhodi/izhodi RC0-7 - LCD D0-7

;*** Uporaba RAMa ***
;20-27	VRSTA vrednosti 8 vrstic
;28	VRSTA skupina 8 vrstic
;29	VRSTA, CAKAJ40, ADCON, STEVIL, DELI, ODDAJA racunanje
;2A	VRSTA, MAX, ODDAJA, PIKICE, SKALA racunanje
;2B	ZANKA stevec period zage
;2C-2F	ADCON rezultat 2x16bit, ZAPIS 4 stevilke
;30,31	RESMAXn maksimum in njegov polozaj
;32	JAKOST min jakost odboja
;33	VRSTA, VISINA, DELI racunanje
;34,35	VISINA, DELI 16-bitna visina
;36	JAKOST notranje stanje 
;37	SKALA racunanje
;38-4F	STEVIL vmesni pomnilnik
;50-55	RESMAXn, MAXINC maksimumi in njihovi polozaji

;*** Zacetek programa ***
	LIST	P=16F876
	ORG	0
ZACETEK	CALL	INIPIC	;inicializacija PIC 16F876/16F876A
	CALL	INILCD	;vkljuci LCD oba kontrolerja
	CALL	EERD	;precitaj EEPROM v (32)
	MOVWF	32
	CLRF	36	;pocisti zacetno stanje JAKOST
	GOTO	ZANKA

;*** Inicializacija PIC 16F876/16F876A ***
INIPIC	CLRF	5	;zacetno stanje vrat
	CLRF	6
	CLRF	7
	BSF	3,5	;postavi PAGE1
	MOVLW	8F	;OPTION: 1=/RBPU, 0=INTEDG, 0=T0CS, 0=T0SE, 1=PSA, 111=PS2-0
	MOVWF	1
	MOVLW	3	;vhoda RA0-1, izhodi RA2-5
	MOVWF	5
	MOVLW	1	;vhod RB0, izhodi RB1-7
	MOVWF	6
	MOVLW	0	;izhodi RC0-7
	MOVWF	7
	MOVLW	84	;ADCON1: 1=ADFM, 000, 0100=PCFG3-0
	MOVWF	1F
	BCF	3,5	;postavi PAGE0
	MOVLW	81	;ADCON0: 10=ADCS1-0, 000=CHS2-0, 0=GO/DONE, 0, 1=ADON 
	MOVWF	1F
	RETURN

;*** Glavna zanka *** (20-27,2B)
ZANKA	CALL	JAKOST	;nastavljanje MIN jakosti s tipko
	CLRF	2B	;vrstici 0 in 1
	CLRF	5
	BCF	6,1
	CALL	RESMAX1	;prepisi in resetiraj MAX1
	CALL	STEVIS	;stevilski zapis visine v vmesnik
	CALL	ODDAJA	;RS-232 oddaja visine za govor
	CALL	VRSTA	;vpisi skupino 8 vrstic
	CALL	MERI	;izmeri skupino 8 vrstic
	CALL	VRSTA	;vpisi skupino 8 vrstic
	CALL	MERI	;izmeri skupino 8 vrstic
	MOVLW	20	;vrstici 2 in 3
	MOVWF	2B
	MOVWF	5
	BCF	6,1
	CALL	CAKAJ40
	CALL	VRSTA	;vpisi skupino 8 vrstic
	CALL	MERI	;izmeri skupino 8 vrstic
	CALL	VRSTA	;vpisi skupino 8 vrstic
	CALL	MERI	;izmeri skupino 8 vrstic
	CLRF	2B	;vrstici 0 in 1
	CLRF	5
	BCF	6,1
	CALL	RESMAX2	;prepisi in resetiraj MAX2
	CALL	STEVIS	;stevilski zapis visine v vmesnik
	CALL	ODDAJA	;RS-232 oddaja visine za govor
	CALL	VRSTA	;vpisi skupino 8 vrstic
	CALL	MERI	;izmeri skupino 8 vrstic
	CALL	VRSTA	;vpisi skupino 8 vrstic
	CALL	MERI	;izmeri skupino 8 vrstic
	MOVLW	40	;vrstici 4 in 5
	MOVWF	2B
	MOVWF	5
	BSF	6,1
	CALL	CAKAJ40
	CALL	VRSTA	;vpisi skupino 8 vrstic
	CALL	MERI	;izmeri skupino 8 vrstic
	CALL	VRSTA	;vpisi skupino 8 vrstic
	CALL	MERI	;izmeri skupino 8 vrstic
	CLRF	2B	;vrstici 0 in 1
	CLRF	5
	BCF	6,1
	CALL	RESMAX3	;prepisi in resetiraj MAX3
	CALL	STEVIS	;stevilski zapis visine v vmesnik
	CALL	ODDAJA	;RS-232 oddaja visine za govor
	CALL	VRSTA	;vpisi skupino 8 vrstic
	CALL	MERI	;izmeri skupino 8 vrstic
	CALL	VRSTA	;vpisi skupino 8 vrstic
	CALL	MERI	;izmeri skupino 8 vrstic
	MOVLW	60	;vrstici 6 in 7
	MOVWF	2B
	MOVWF	5
	BSF	6,1
	CALL	CAKAJ40
	CALL	VRSTA	;vpisi skupino 8 vrstic
	CALL	MERI	;izmeri skupino 8 vrstic
	CALL	VRSTA	;vpisi skupino 8 vrstic
	CALL	MERI	;izmeri skupino 8 vrstic
	GOTO	ZANKA

;*** RS-232 oddaja visine za govor na RB4 *** (29,2A)
ODDAJA	RRF	31,0	;obrezi podatke biti 0-5
	ANDLW	3F
	MOVWF	2A
	MOVF	32,0	;preveri jakost v bit 6
	SUBWF	30,0
	BTFSC	3,0
	BSF	2A,6
	MOVLW	9	;start + 8 podatkov
	MOVWF	29
	BCF	3,0	;start bit
ODDAJA1	BCF	6,4
	BTFSS	3,0
	BSF	6,4
	CALL	CAKAJB
	RRF	2A,1
	DECFSZ	29,1
	GOTO	ODDAJA1
	BCF	6,4	;stop biti
	RETURN

;*** Cakaj en bit 9.6kbps (104us), 20MHz >>> 521 ukazov ***
CAKAJB	MOVLW	7F	;127 zank + call/return (4) + ostali ukazi (9)
	NOP
CAKAJB1	ADDLW	0FF	;zanka 4 ukazi
	BTFSS	3,2
	GOTO	CAKAJB1
	RETURN

;*** Nastavljanje MIN jakosti s tipko *** (32,36)
JAKOST	MOVLW	7F	;preveri MIN jakost
	ANDWF	32,1
	BTFSS	6,0	;tipka RB0?
	GOTO	JAKOST1
	MOVF	36,0	;preglej stevec >1?
	ANDLW	7F
	ADDLW	0FF
	BTFSS	3,0
	GOTO	JAKOST3
	COMF	36,1	;obrni bit 7 za smer
	CALL	EERD	;primerjaj EEPROM in (32)?
	SUBWF	32,0
	BTFSC	3,2
	GOTO	JAKOST3
	MOVF	32,0	;vpisi novo v EEPROM
	CALL	EEWR
JAKOST3	MOVLW	80	;pobrisi stevec biti 0-6
	ANDWF	36,1
	RETURN
JAKOST1 MOVF	36,0	;preglej stevec >6?
	ANDLW	7F
	ADDLW	0FA
	BTFSS	3,0
	GOTO	JAKOST2
	MOVLW	1	;smer nastavljanja?
	BTFSC	36,7
	MOVLW	0FF
	ADDWF	32,1	;popravi MIN jakost
JAKOST2	INCF	36,1	;povecaj in zaustavi stevec
	BTFSC	36,6
	BCF	36,5
	RETURN

;*** Precitaj EEPROM v (W) ***
EERD	BSF	3,6	;postavi PAGE2
	MOVLW	3	;EEADR=3
	MOVWF	0D
	BSF	3,5	;postavi PAGE3 - precitaj (32) iz EEPROMa
	BCF	0C,7	;EECON1: EEPGD=0 data memory
	BSF	0C,0	;EECON1: RD
	BCF	3,5	;postavi PAGE2
	MOVF	0C,0	;precitaj EEDATA
	BCF	3,6	;postavi PAGE0
	RETURN

;*** Vpisi (W) v EEPROM ***
EEWR	BSF	3,6	;postavi PAGE2
	MOVWF	0C	;EEDATA
	MOVLW	3	;EEADR=3
	MOVWF	0D
	BSF	3,5	;postavi PAGE3 - precitaj (32) iz EEPROMa
	BCF	0C,7	;EECON1: EEPGD=0 data memory
	BSF	0C,2	;EECON1: WREN=1
	MOVLW	55	;EECON2=$55
	MOVWF	0D
	MOVLW	0AA	;EECON2=$AA
	MOVWF	0D
	BSF	0C,1	;EECON1: WR=1
EEWR1	BTFSC	0C,1	;EECON1 preveri WR za konec?
	GOTO	EEWR1
	BCF	0C,2	;EECON1: WREN=0
	BCF	3,6	;postavi PAGE0
	BCF	3,5
	RETURN

;*** Izmeri skupino 8 vrstic *** (20-27)
MERI	CALL	ADCON	;povprecenje 2x64 meritev
	MOVF	2C,0
	MOVWF	27
	CALL	MAXINC
	MOVF	2E,0
	MOVWF	26
	CALL	MAXINC
	MOVF	2B,0	;postavi novo periodo zage
	MOVWF	5
	CALL	ADCON	;povprecenje 2x64 meritev
	MOVF	2C,0
	MOVWF	25
	CALL	MAXINC
	MOVF	2E,0
	MOVWF	24
	CALL	MAXINC
	MOVF	2B,0	;postavi novo periodo zage
	MOVWF	5
	CALL	ADCON	;povprecenje 2x64 meritev
	MOVF	2C,0
	MOVWF	23
	CALL	MAXINC
	MOVF	2E,0
	MOVWF	22
	CALL	MAXINC
	MOVF	2B,0	;postavi novo periodo zage
	MOVWF	5
	CALL	ADCON	;povprecenje 2x64 meritev
	MOVF	2C,0
	MOVWF	21
	CALL	MAXINC
	MOVF	2E,0
	MOVWF	20
	CALL	MAXINC
	MOVF	2B,0	;postavi novo periodo zage
	MOVWF	5
	RETURN

;*** Poisci maksimum W>(50,52,54) na visini (2B)>(51,53,55) *** (2A,2B,50-55)
MAXINC	MOVWF	2A	;primerjaj s starim maksimumom v (50)
	SUBWF	50,0
	BTFSC	3,0	
	GOTO	MAXINC1
	MOVF	2A,0	;shrani nov maksimum v (50), polozaj v (51)
	MOVWF	50
	MOVF	2B,0
	MOVWF	51
MAXINC1	MOVF	2A,0	;primerjaj s starim maksimumom v (52)
	SUBWF	52,0
	BTFSC	3,0	
	GOTO	MAXINC2
	MOVF	2A,0	;shrani nov maksimum v (52), polozaj v (53)
	MOVWF	52
	MOVF	2B,0
	MOVWF	53
MAXINC2	MOVF	2A,0	;primerjaj s starim maksimumom v (54)
	SUBWF	54,0
	BTFSC	3,0	
	GOTO	MAXINC3
	MOVF	2A,0	;shrani nov maksimum v (54), polozaj v (55)
	MOVWF	54
	MOVF	2B,0
	MOVWF	55
MAXINC3	INCF	2B,1	;povecaj periodo zage (2B)
	INCF	2B,1
	RETURN

;*** Prepisi in resetiraj MAX 1,2,3 *** (30,31,50-55)
RESMAX1	MOVF	50,0	;prepisi maksimum 1 v (30,31)
	MOVWF	30
	MOVF	51,0
	MOVWF	31
	CLRF	50	;reset maksimuma 1 v (50,51)
	CLRF	51
	RETURN
RESMAX2	MOVF	52,0	;prepisi maksimum 2 v (30,31)
	MOVWF	30
	MOVF	53,0
	MOVWF	31
	CLRF	52	;reset maksimuma 2 v (52,53)
	CLRF	53
	RETURN
RESMAX3	MOVF	54,0	;prepisi maksimum 3 v (30,31)
	MOVWF	30
	MOVF	55,0
	MOVWF	31
	CLRF	54	;reset maksimuma 3 v (54,55)
	CLRF	55
	RETURN

;*** Cakaj priblizno 30us (takt 20MHz) ***
CAKAJP	MOVLW	24
CAKAJP1	ADDLW	0FF	;zanka 4 ukazi
	BTFSS	3,2
	GOTO	CAKAJP1
	RETURN

;*** Cakanje 40XCAKAJP *** (29)
CAKAJ40	MOVLW	28
	MOVWF	29
CAKAJ41	CALL	CAKAJP
	DECFSZ	29,1
	GOTO	CAKAJ41
	RETURN

;*** Povprecenje 2x64 vzorcev A/D pretvornika *** (29,2B,2C,2D,2E,2F)
ADCON	CALL	CAKAJ40	;pocakaj, da se rezultat umiri
	CLRF	2C
	CLRF	2D
	CLRF	2E
	CLRF	2F
	MOVLW	40
	MOVWF	29
ADCON1	BCF	1F,3	;vzorcenje kanala 0
	CALL	CAKAJP
	BSF	1F,2	;zacni novo A/D pretvorbo
ADCON2	BTFSC	1F,2	;cakaj na konec A/D pretvorbe
	GOTO	ADCON2
	BSF	3,5	;postavi PAGE1
	MOVF	1E,0	;spodnji rezultat A/D pretvorbe
	BCF	3,5	;postavi PAGE0
	ADDWF	2D,1
	BTFSC	3,0
	INCF	2C,1
	MOVF	1E,0	;gornji rezultat A/D pretvorbe
	ADDWF	2C,1
	BSF	1F,3	;vzorcenje kanala 1
	CALL	CAKAJP
	BSF	1F,2	;zacni novo A/D pretvorbo
	BSF	1F,2	;zacni novo A/D pretvorbo
ADCON3	BTFSC	1F,2	;cakaj na konec A/D pretvorbe
	GOTO	ADCON3
	BSF	3,5	;postavi PAGE1
	MOVF	1E,0	;spodnji rezultat A/D pretvorbe
	BCF	3,5	;postavi PAGE0
	ADDWF	2F,1
	BTFSC	3,0
	INCF	2E,1
	MOVF	1E,0	;gornji rezultat A/D pretvorbe
	ADDWF	2E,1
	DECFSZ	29,1
	GOTO	ADCON1
	MOVF	2B,0	;izravnava ojacenja sorazmerno z zago
	ANDLW	7C
	ADDLW	80
	ADDWF	2C,1	;sodi kanal 0
	BTFSS	3,0
	CLRF	2C
	ADDLW	2	;vmesni korak za lihe kanale
	ADDWF	2E,1	;lihi kanal 1
	BTFSS	3,0
	CLRF	2E
	RETURN

;*** Vpisi skupino 8 vrstic *** (20-27,28,29,2A,33)
VRSTA	INCF	32,0	;izpis min jakosti
	MOVWF	33
	BCF	6,2	;0=/CS1 prva polovica zaslona
	BSF	6,3	;1=/CS2
	CALL	VRSTA1
	BSF	6,2	;1=/CS1 druga polovica zaslona
	BCF	6,3	;0=/CS2
	CALL	VRSTA1
	COMF	2B,0	;izracunaj skupino 8 vrstic naslednje vrste
	MOVWF	28
	RRF	28,1
	RRF	28,1
	RRF	28,1
	RRF	28,1
	RETURN
VRSTA1	MOVF	28,0	;izberi skupino 8 vrstic
	ANDLW	7
	IORLW	0B8
	CALL	UKAZ
	MOVLW	40	;izberi prvi stolpec
	CALL	UKAZ
	MOVLW	40	;zanka 64 stoplcev
	MOVWF	29
VRSTA2	CLRF	2A
	DECF	20,1	;izracun 8 vrstic
	BTFSS	20,7
	BSF	2A,0
	DECF	21,1
	BTFSS	21,7
	BSF	2A,1
	DECF	22,1
	BTFSS	22,7
	BSF	2A,2
	DECF	23,1
	BTFSS	23,7
	BSF	2A,3
	DECF	24,1
	BTFSS	24,7
	BSF	2A,4
	DECF	25,1
	BTFSS	25,7
	BSF	2A,5
	DECF	26,1
	BTFSS	26,7
	BSF	2A,6
	DECF	27,1
	BTFSS	27,7
	BSF	2A,7
	CALL	PIKICE	;pikice za skalo na zaslonu
	DECF	33,1	;izpis min jakosti s tipko RB0?
	BTFSS	6,0
	CALL	PALCKA
	MOVF	2A,0
	CALL	BAJT	;LCD podatek = 8-bitni odsek
	DECFSZ	29,1
	GOTO	VRSTA2
	RETURN

;*** Izpis palcke na zaslonu za MIN jakost *** (2A)
PALCKA	BTFSS	36,7	;izpis smeri navzdol?
	GOTO	PALCKA1
	DECF	33,0
	BTFSS	3,2
	GOTO	PALCKA1
	MOVLW	18
	XORWF	2A,1
PALCKA1	BTFSC	36,7	;izpis smeri navzgor?
	GOTO	PALCKA2
	INCF	33,0
	BTFSS	3,2
	GOTO	PALCKA2
	MOVLW	18
	XORWF	2A,1
PALCKA2	MOVF	33,0	;izpis palcke?
	BTFSC	3,2
	COMF	2A,1
	RETURN

;*** Pikice za skalo na zaslonu *** (2A)
PIKICE	BTFSC	6,3	;druga polovica zaslona?
	GOTO	PIKICEP
	COMF	28,0	;spodnja skupina vrstic?
	ANDLW	7
	BTFSC	3,2
	GOTO	PIKICES
	MOVLW	10	;zadnjih 16 stolpcev?
	SUBWF	29,0
	BTFSC	3,0
	GOTO	PIKICEP
	GOTO	SKALA
PIKICES	MOVLW	18	;zadnjih 24 stolpcev?
	SUBWF	29,0
	BTFSC	3,0
	GOTO	PIKICEP
	MOVLW	50	;zapis za podlago
	MOVWF	4
	MOVF	29,0
	SUBWF	4,1
	MOVF	0,0
	XORWF	2A,1
	RETURN
PIKICEP DECF	29,0	;vsak 16. stolpec pikic?
	ANDLW	0F
	BTFSS	3,2
	RETURN
	COMF	28,0	;stevilka skupine vrstic
	ANDLW	7
	CALL	PIKICE0
	XORWF	2A,1
	RETURN
PIKICE0	ADDLW	0
	BTFSS	3,2
	GOTO	PIKICE1
	MOVLW	1	;10ft(+12ft)
	RETURN
PIKICE1	ADDLW	0FF
	BTFSS	3,2
	GOTO	PIKICE2
	MOVLW	2	;30ft(+12ft)
	RETURN
PIKICE2	ADDLW	0FF
	BTFSS	3,2
	GOTO	PIKICE3
	MOVLW	1	;100ft(+12ft)
	RETURN
PIKICE3	ADDLW	0FF
	BTFSS	3,2
	GOTO	PIKICE4
	MOVLW	0	;prazno
	RETURN
PIKICE4	ADDLW	0FF
	BTFSS	3,2
	GOTO	PIKICE5
	MOVLW	20	;300ft
	RETURN
PIKICE5	ADDLW	0FF
	BTFSS	3,2
	GOTO	PIKICE6
	MOVLW	2	;1000ft
	RETURN
PIKICE6	ADDLW	0FF
	BTFSS	3,2
	GOTO	PIKICE7
	MOVLW	0	;prazno
	RETURN
PIKICE7	MOVLW	20	;3000ft
	RETURN

;*** Stevilska skala v zadnjih 16 stolpcih *** (2A,37)
SKALA	COMF	28,0	;stevilka skupine vrstic
	ANDLW	7
	MOVWF	37
	MOVF	29,0	;stevilka stolpca
	DECF	37,1	;vrstica1
	BTFSS	3,2
	GOTO	SKALA1
	MOVWF	37
	MOVLW	0
	DECF	37,1	;0
	BTFSC	3,2
	MOVLW	0F
	DECF	37,1
	BTFSC	3,2
	MOVLW	8
	DECF	37,1
	BTFSC	3,2
	MOVLW	0F
	DECF	37,1
	DECF	37,1	;3
	BTFSC	3,2
	MOVLW	0F
	DECF	37,1
	BTFSC	3,2
	MOVLW	0A
	DECF	37,1
	BTFSC	3,2
	MOVLW	0A
	XORWF	2A,1
	RETURN
SKALA1	DECF	37,1	;vrstica2
	BTFSS	3,2
	GOTO	SKALA2
	MOVWF	37
	MOVLW	0
	DECF	37,1	;0+0
	BTFSC	3,2
	MOVLW	87
	DECF	37,1
	BTFSC	3,2
	MOVLW	84
	DECF	37,1
	BTFSC	3,2
	MOVLW	87
	DECF	37,1
	DECF	37,1	;0+3
	BTFSC	3,2
	MOVLW	87
	DECF	37,1
	BTFSC	3,2
	MOVLW	84
	DECF	37,1
	BTFSC	3,2
	MOVLW	87
	DECF	37,1
	DECF	37,1	;1
	BTFSC	3,2
	MOVLW	7
	XORWF	2A,1
	RETURN
SKALA2	DECF	37,1	;vrstica3
	BTFSS	3,2
	GOTO	SKALA3
	MOVWF	37
	MOVLW	0
	DECF	37,1	;0
	BTFSC	3,2
	MOVLW	0C0
	DECF	37,1
	BTFSC	3,2
	MOVLW	40
	DECF	37,1
	BTFSC	3,2
	MOVLW	0C0
	DECF	37,1
	DECF	37,1	;0
	BTFSC	3,2
	MOVLW	0C0
	DECF	37,1
	BTFSC	3,2
	MOVLW	40
	DECF	37,1
	BTFSC	3,2
	MOVLW	0C0
	DECF	37,1
	DECF	37,1	;1
	BTFSC	3,2
	MOVLW	0C0
	XORWF	2A,1
	RETURN
SKALA3	DECF	37,1	;vrstica4
	BTFSS	3,2
	GOTO	SKALA4
	MOVWF	37
	MOVLW	0
	DECF	37,1	;0
	BTFSC	3,2
	MOVLW	0F8
	DECF	37,1
	BTFSC	3,2
	MOVLW	88
	DECF	37,1
	BTFSC	3,2
	MOVLW	0F8
	DECF	37,1
	DECF	37,1	;0
	BTFSC	3,2
	MOVLW	0F8
	DECF	37,1
	BTFSC	3,2
	MOVLW	88
	DECF	37,1
	BTFSC	3,2
	MOVLW	0F8
	DECF	37,1
	DECF	37,1	;3
	BTFSC	3,2
	MOVLW	0F8
	DECF	37,1
	BTFSC	3,2
	MOVLW	0A8
	DECF	37,1
	BTFSC	3,2
	MOVLW	0A8
	XORWF	2A,1
	RETURN
SKALA4	DECF	37,1	;vrstica5
	BTFSS	3,2
	GOTO	SKALA5
	MOVWF	37
	MOVLW	0
	DECF	37,1	;0
	BTFSC	3,2
	MOVLW	0F
	DECF	37,1
	BTFSC	3,2
	MOVLW	8
	DECF	37,1
	BTFSC	3,2
	MOVLW	0F
	DECF	37,1
	DECF	37,1	;0
	BTFSC	3,2
	MOVLW	0F
	DECF	37,1
	BTFSC	3,2
	MOVLW	8
	DECF	37,1
	BTFSC	3,2
	MOVLW	0F
	DECF	37,1
	DECF	37,1	;0
	BTFSC	3,2
	MOVLW	0F
	DECF	37,1
	BTFSC	3,2
	MOVLW	8
	DECF	37,1
	BTFSC	3,2
	MOVLW	0F
	DECF	37,1
	DECF	37,1	;1
	BTFSC	3,2
	MOVLW	0F
	XORWF	2A,1
	RETURN
SKALA5	DECF	37,1	;vrstica6
	BTFSS	3,2
	GOTO	SKALA6
	MOVWF	37
	MOVLW	0
	DECF	37,1	;0
	BTFSC	3,2
	MOVLW	80
	DECF	37,1
	BTFSC	3,2
	MOVLW	80
	DECF	37,1
	BTFSC	3,2
	MOVLW	80
	DECF	37,1
	DECF	37,1	;0
	BTFSC	3,2
	MOVLW	80
	DECF	37,1
	BTFSC	3,2
	MOVLW	80
	DECF	37,1
	BTFSC	3,2
	MOVLW	80
	DECF	37,1
	DECF	37,1	;0
	BTFSC	3,2
	MOVLW	80
	DECF	37,1
	BTFSC	3,2
	MOVLW	80
	DECF	37,1
	BTFSC	3,2
	MOVLW	80
	DECF	37,1
	DECF	37,1	;1
	BTFSC	3,2
	MOVLW	80
	XORWF	2A,1
	RETURN
SKALA6	MOVWF	37	;vrstica7
	MOVLW	0
	DECF	37,1	;0
	BTFSC	3,2
	MOVLW	0F8
	DECF	37,1
	BTFSC	3,2
	MOVLW	88
	DECF	37,1
	BTFSC	3,2
	MOVLW	0F8
	DECF	37,1
	DECF	37,1	;0
	BTFSC	3,2
	MOVLW	0F8
	DECF	37,1
	BTFSC	3,2
	MOVLW	88
	DECF	37,1
	BTFSC	3,2
	MOVLW	0F8
	DECF	37,1
	DECF	37,1	;0
	BTFSC	3,2
	MOVLW	0F8
	DECF	37,1
	BTFSC	3,2
	MOVLW	88
	DECF	37,1
	BTFSC	3,2
	MOVLW	0F8
	DECF	37,1
	DECF	37,1	;3
	BTFSC	3,2
	MOVLW	0F8
	DECF	37,1
	BTFSC	3,2
	MOVLW	0A8
	DECF	37,1
	BTFSC	3,2
	MOVLW	0A8
	XORWF	2A,1
	RETURN

;*** Vkljuci in inicializiraj LCD oba kontrolerja ***
INILCD	BSF	6,2	;1=/CS1
	BCF	6,3	;0=/CS2
	MOVLW	3F	;display ON
	CALL	UKAZ
	MOVLW	0C0	;display start line 0
	CALL	UKAZ
	BCF	6,2	;0=/CS1
	BSF	6,3	;1=/CS2
	MOVLW	3F	;display ON
	CALL	UKAZ
	MOVLW	0C0	;display start line 0
	CALL	UKAZ
	RETURN

;*** Preveri BUSY in poslji podatek iz W ***
BAJT	BCF	6,7	;0=enable
	BSF	6,5	;1=read
	BCF	6,6	;0=instruction
	BSF	6,7	;1=enable
	BSF	3,5	;postavi PAGE1
	CLRF	7
	COMF	7,1	;vhodi RC0-7
	BCF	3,5	;postavi PAGE0
	BTFSC	7,7	;preveri D7=BUSY
	GOTO	BAJT
	BCF	6,7	;0=enable
	BSF	3,5	;postavi PAGE1
	CLRF	7	;izhodi RC0-7
	BCF	3,5	;postavi PAGE0
	MOVWF	7	;W na D0-7
	BCF	6,5	;0=write
	BSF	6,6	;1=data
	BSF	6,7	;1=enable
	NOP
	NOP
	BCF	6,7	;0=enable
	RETURN

;*** Preveri BUSY in poslji ukaz iz W ***
UKAZ	BCF	6,7	;0=enable
	BSF	6,5	;1=read
	BCF	6,6	;0=instruction
	BSF	6,7	;1=enable
	BSF	3,5	;postavi PAGE1
	CLRF	7
	COMF	7,1	;vhodi RC0-7
	BCF	3,5	;postavi PAGE0
	BTFSC	7,7	;preveri D7=BUSY
	GOTO	UKAZ
	BCF	6,7	;0=enable
	BSF	3,5	;postavi PAGE1
	CLRF	7	;izhodi RC0-7
	BCF	3,5	;postavi PAGE0
	MOVWF	7	;W na D0-7
	BCF	6,5	;0=write
	BSF	6,7	;1=enable
	NOP
	NOP
	BCF	6,7	;0=enable
	RETURN

;*** Stevilski zapis visine v vmesnik *** (2C-2F,38-4F)
STEVIS	CALL	VISINA	;preracunaj max periodo (31) v visino (34,35)
	CALL	DELI	;pretvori visino v 4-mestno (2C-2F)
	MOVWF	2F
	CALL	DELI
	MOVWF	2E
	CALL	DELI
	MOVWF	2D
	CALL	DELI
	MOVWF	2C
	MOVLW	0A	;pobrisi vodilne nicle, 10 za praznino
	MOVF	2C,1
	BTFSS	3,2
	GOTO	ZAPIS1
	MOVWF	2C
	MOVF	2D,1
	BTFSS	3,2
	GOTO	ZAPIS1
	MOVWF	2D
	MOVF	2E,1
	BTFSS	3,2
	GOTO	ZAPIS1
	MOVWF	2E
ZAPIS1	MOVLW	38	;zapis v vmesnik (38-4F)
	MOVWF	4
	MOVF	2C,0
	CALL	STEVIL
	MOVF	2D,0
	CALL	STEVIL
	MOVF	2E,0
	CALL	STEVIL
	MOVF	2F,0
	CALL	STEVIL
	RETURN

;*** Deli (34,35) z 10 in ostanek v (33) in W *** (29,33,34,35)
DELI	CLRF	33
	MOVLW	10
	MOVWF	29
DELI1	RLF	35,1	;premakni vse za 1bit v levo
	RLF	34,1
	RLF	33,1
	BCF	35,0
	MOVLW	0A	;odstej 10
	SUBWF	33,0
	BTFSS	3,0
	GOTO	DELI2
	MOVWF	33	;popravi ostanek in kvocient
	BSF	35,0
DELI2	DECFSZ	29,1
	GOTO	DELI1
	MOVF	33,0
	RETURN

;*** Izpisi stevilko iz W *** (29)
STEVIL	MOVWF	29
	MOVLW	0	;praznina do prejsnje stevilke
	CALL	STEVILB
	MOVF	29,1
	BTFSS	3,2
	GOTO	STEVIL1
	MOVLW	0FE	;"0" FE 82 82 82 FE
	CALL	STEVILB
	MOVLW	82
	CALL	STEVILB
	MOVLW	82
	CALL	STEVILB
	MOVLW	82
	CALL	STEVILB
	MOVLW	0FE
	CALL	STEVILB
	RETURN
STEVIL1	DECF	29,1
	BTFSS	3,2
	GOTO	STEVIL2
	MOVLW	0	;"1" 00 00 00 00 FE
	CALL	STEVILB
	MOVLW	0
	CALL	STEVILB
	MOVLW	0
	CALL	STEVILB
	MOVLW	0
	CALL	STEVILB
	MOVLW	0FE
	CALL	STEVILB
	RETURN
STEVIL2	DECF	29,1
	BTFSS	3,2
	GOTO	STEVIL3
	MOVLW	0F2	;"2" F2 92 92 92 9E
	CALL	STEVILB
	MOVLW	92
	CALL	STEVILB
	MOVLW	92
	CALL	STEVILB
	MOVLW	92
	CALL	STEVILB
	MOVLW	9E
	CALL	STEVILB
	RETURN
STEVIL3	DECF	29,1
	BTFSS	3,2
	GOTO	STEVIL4
	MOVLW	92	;"3" 92 92 92 92 FE
	CALL	STEVILB
	MOVLW	92
	CALL	STEVILB
	MOVLW	92
	CALL	STEVILB
	MOVLW	92
	CALL	STEVILB
	MOVLW	0FE
	CALL	STEVILB
	RETURN
STEVIL4	DECF	29,1
	BTFSS	3,2
	GOTO	STEVIL5
	MOVLW	1E	;"4" 1E 10 10 10 FE
	CALL	STEVILB
	MOVLW	10
	CALL	STEVILB
	MOVLW	10
	CALL	STEVILB
	MOVLW	10
	CALL	STEVILB
	MOVLW	0FE
	CALL	STEVILB
	RETURN
STEVIL5	DECF	29,1
	BTFSS	3,2
	GOTO	STEVIL6
	MOVLW	9E	;"5" 9E 92 92 92 F2
	CALL	STEVILB
	MOVLW	92
	CALL	STEVILB
	MOVLW	92
	CALL	STEVILB
	MOVLW	92
	CALL	STEVILB
	MOVLW	0F2
	CALL	STEVILB
	RETURN
STEVIL6	DECF	29,1
	BTFSS	3,2
	GOTO	STEVIL7
	MOVLW	0FE	;"6" FE 90 90 90 F0
	CALL	STEVILB
	MOVLW	90
	CALL	STEVILB
	MOVLW	90
	CALL	STEVILB
	MOVLW	90
	CALL	STEVILB
	MOVLW	0F0
	CALL	STEVILB
	RETURN
STEVIL7	DECF	29,1
	BTFSS	3,2
	GOTO	STEVIL8
	MOVLW	02	;"7" 02 02 02 02 FE
	CALL	STEVILB
	MOVLW	02
	CALL	STEVILB
	MOVLW	02
	CALL	STEVILB
	MOVLW	02
	CALL	STEVILB
	MOVLW	0FE
	CALL	STEVILB
	RETURN
STEVIL8	DECF	29,1
	BTFSS	3,2
	GOTO	STEVIL9
	MOVLW	0FE	;"8" FE 92 92 92 FE
	CALL	STEVILB
	MOVLW	92
	CALL	STEVILB
	MOVLW	92
	CALL	STEVILB
	MOVLW	92
	CALL	STEVILB
	MOVLW	0FE
	CALL	STEVILB
	RETURN
STEVIL9	DECF	29,1
	BTFSS	3,2
	GOTO	STEVILZ
	MOVLW	1E	;"9" 1E 12 12 12 FE
	CALL	STEVILB
	MOVLW	12
	CALL	STEVILB
	MOVLW	12
	CALL	STEVILB
	MOVLW	12
	CALL	STEVILB
	MOVLW	0FE
	CALL	STEVILB
	RETURN
STEVILZ	MOVLW	0	;"praznina" 00 00 00 00 00
	CALL	STEVILB
	MOVLW	0
	CALL	STEVILB
	MOVLW	0
	CALL	STEVILB
	MOVLW	0
	CALL	STEVILB
	MOVLW	0
	CALL	STEVILB
	RETURN
STEVILB	MOVWF	0	;bajt v vmesni pomnilnik
	INCF	4,1	;povecaj kazalec (4)
	RETURN

;*** Pretvori periodo (31) v visino (34,35) *** (33,34,35)
VISINA	CLRF	34	;0 - visina 0ft $0000
	CLRF	35
	RRF	31,0
	ANDLW	3F
	BTFSC	3,2
	RETURN
	MOVWF	33
	MOVLW	1	;1 - visina 1ft $0001
	MOVWF	35
	DECF	33,1
	BTFSC	3,2
	RETURN
	MOVLW	2	;2 - visina 1ft $0002
	MOVWF	35
	DECF	33,1
	BTFSC	3,2
	RETURN
	MOVLW	4	;3 - visina 4ft $0004
	MOVWF	35
	DECF	33,1
	BTFSC	3,2
	RETURN
	MOVLW	5	;4 - visina 5ft $0005
	MOVWF	35
	DECF	33,1
	BTFSC	3,2
	RETURN
	MOVLW	7	;5 - visina 4ft $0007
	MOVWF	35
	DECF	33,1
	BTFSC	3,2
	RETURN
	MOVLW	0A	;6 - visina 10ft $000A
	MOVWF	35
	DECF	33,1
	BTFSC	3,2
	RETURN
	MOVLW	0C	;7 - visina 12ft $000C
	MOVWF	35
	DECF	33,1
	BTFSC	3,2
	RETURN
	MOVLW	0E	;8 - visina 14ft $000E
	MOVWF	35
	DECF	33,1
	BTFSC	3,2
	RETURN
	MOVLW	10	;9 - visina 16ft $0010
	MOVWF	35
	DECF	33,1
	BTFSC	3,2
	RETURN
	MOVLW	13	;10 - visina 19ft $0013
	MOVWF	35
	DECF	33,1
	BTFSC	3,2
	RETURN
	MOVLW	16	;11 - visina 22ft $0016
	MOVWF	35
	DECF	33,1
	BTFSC	3,2
	RETURN
	MOVLW	1A	;12 - visina 26ft $001A
	MOVWF	35
	DECF	33,1
	BTFSC	3,2
	RETURN
	MOVLW	1E	;13 - visina 30ft $001E
	MOVWF	35
	DECF	33,1
	BTFSC	3,2
	RETURN
	MOVLW	23	;14 - visina 35ft $0023
	MOVWF	35
	DECF	33,1
	BTFSC	3,2
	RETURN
	MOVLW	28	;15 - visina 40ft $0028
	MOVWF	35
	DECF	33,1
	BTFSC	3,2
	RETURN
	MOVLW	2D	;16 - visina 45ft $002D
	MOVWF	35
	DECF	33,1
	BTFSC	3,2
	RETURN
	MOVLW	32	;17 - visina 50ft $0032
	MOVWF	35
	DECF	33,1
	BTFSC	3,2
	RETURN
	MOVLW	37	;18 - visina 55ft $0037
	MOVWF	35
	DECF	33,1
	BTFSC	3,2
	RETURN
	MOVLW	3C	;19 - visina 60ft $003C
	MOVWF	35
	DECF	33,1
	BTFSC	3,2
	RETURN
	MOVLW	46	;20 - visina 70ft $0046
	MOVWF	35
	DECF	33,1
	BTFSC	3,2
	RETURN
	MOVLW	50	;21 - visina 80ft $0050
	MOVWF	35
	DECF	33,1
	BTFSC	3,2
	RETURN
	MOVLW	5A	;22 - visina 90ft $005A
	MOVWF	35
	DECF	33,1
	BTFSC	3,2
	RETURN
	MOVLW	64	;23 - visina 100ft $0064
	MOVWF	35
	DECF	33,1
	BTFSC	3,2
	RETURN
	MOVLW	6E	;24 - visina 110ft $006E
	MOVWF	35
	DECF	33,1
	BTFSC	3,2
	RETURN
	MOVLW	78	;25 - visina 120ft $0078
	MOVWF	35
	DECF	33,1
	BTFSC	3,2
	RETURN
	MOVLW	82	;26 - visina 130ft $0082
	MOVWF	35
	DECF	33,1
	BTFSC	3,2
	RETURN
	MOVLW	96	;27 - visina 150ft $0096
	MOVWF	35
	DECF	33,1
	BTFSC	3,2
	RETURN
	MOVLW	0AA	;28 - visina 170ft $00AA
	MOVWF	35
	DECF	33,1
	BTFSC	3,2
	RETURN
	MOVLW	0B4	;29 - visina 180ft $00B4
	MOVWF	35
	DECF	33,1
	BTFSC	3,2
	RETURN
	MOVLW	0C8	;30 - visina 200ft $00C8
	MOVWF	35
	DECF	33,1
	BTFSC	3,2
	RETURN
	MOVLW	0DC	;31 - visina 220ft $00DC
	MOVWF	35
	DECF	33,1
	BTFSC	3,2
	RETURN
	MOVLW	0FA	;32 - visina 250ft $00FA
	MOVWF	35
	DECF	33,1
	BTFSC	3,2
	RETURN
	INCF	34,1	;33 - visina 270ft $010E
	MOVLW	0E
	MOVWF	35
	DECF	33,1
	BTFSC	3,2
	RETURN
	MOVLW	2C	;34 - visina 300ft $012C
	MOVWF	35
	DECF	33,1
	BTFSC	3,2
	RETURN
	MOVLW	4A	;35 - visina 330ft $014A
	MOVWF	35
	DECF	33,1
	BTFSC	3,2
	RETURN
	MOVLW	72	;36 - visina 370ft $0172
	MOVWF	35
	DECF	33,1
	BTFSC	3,2
	RETURN
	MOVLW	90	;37 - visina 400ft $0190
	MOVWF	35
	DECF	33,1
	BTFSC	3,2
	RETURN
	MOVLW	0C2	;38 - visina 450ft $01C2
	MOVWF	35
	DECF	33,1
	BTFSC	3,2
	RETURN
	MOVLW	0F4	;39 - visina 500ft $01F4
	MOVWF	35
	DECF	33,1
	BTFSC	3,2
	RETURN
	INCF	34,1	;40 - visina 550ft $0226
	MOVLW	26
	MOVWF	35
	DECF	33,1
	BTFSC	3,2
	RETURN
	MOVLW	58	;41 - visina 600ft $0258
	MOVWF	35
	DECF	33,1
	BTFSC	3,2
	RETURN
	MOVLW	94	;42 - visina 660ft $0294
	MOVWF	35
	DECF	33,1
	BTFSC	3,2
	RETURN
	MOVLW	0DA	;43 - visina 730ft $02DA
	MOVWF	35
	DECF	33,1
	BTFSC	3,2
	RETURN
	INCF	34,1	;44 - visina 800ft $0320
	MOVLW	20
	MOVWF	35
	DECF	33,1
	BTFSC	3,2
	RETURN
	MOVLW	84	;45 - visina 900ft $0384
	MOVWF	35
	DECF	33,1
	BTFSC	3,2
	RETURN
	MOVLW	0E8	;46 - visina 1000ft $03E8
	MOVWF	35
	DECF	33,1
	BTFSC	3,2
	RETURN
	INCF	34,1	;47 - visina 1100ft $044C
	MOVLW	4C
	MOVWF	35
	DECF	33,1
	BTFSC	3,2
	RETURN
	MOVLW	0B0	;48 - visina 1200ft $04B0
	MOVWF	35
	DECF	33,1
	BTFSC	3,2
	RETURN
	INCF	34,1	;49 - visina 1300ft $0514
	MOVLW	14
	MOVWF	35
	DECF	33,1
	BTFSC	3,2
	RETURN
	MOVLW	78	;50 - visina 1400ft $0578
	MOVWF	35
	DECF	33,1
	BTFSC	3,2
	RETURN
	MOVLW	0DC	;51 - visina 1500ft $05DC
	MOVWF	35
	DECF	33,1
	BTFSC	3,2
	RETURN
	INCF	34,1	;52 - visina 1700ft $06A4
	MOVLW	0A4
	MOVWF	35
	DECF	33,1
	BTFSC	3,2
	RETURN
	INCF	34,1	;53 - visina 1900ft $076C
	MOVLW	6C
	MOVWF	35
	DECF	33,1
	BTFSC	3,2
	RETURN
	INCF	34,1	;54 - visina 2100ft $0834
	MOVLW	34
	MOVWF	35
	DECF	33,1
	BTFSC	3,2
	RETURN
	MOVLW	0FC	;55 - visina 2300ft $08FC
	MOVWF	35
	DECF	33,1
	BTFSC	3,2
	RETURN
	INCF	34,1	;56 - visina 2500ft $09C4
	MOVLW	0C4
	MOVWF	35
	DECF	33,1
	BTFSC	3,2
	RETURN
	INCF	34,1	;57 - visina 2800ft $0AF0
	MOVLW	0F0
	MOVWF	35
	DECF	33,1
	BTFSC	3,2
	RETURN
	MOVLW	0C	;58 - visina 3100ft $0C1C
	MOVWF	34
	MOVLW	1C
	MOVWF	35
	DECF	33,1
	BTFSC	3,2
	RETURN
	MOVLW	0D	;59 - visina 3400ft $0D48
	MOVWF	34
	MOVLW	48
	MOVWF	35
	DECF	33,1
	BTFSC	3,2
	RETURN
	MOVLW	0E	;60 - visina 3700ft $0E74
	MOVWF	34
	MOVLW	74
	MOVWF	35
	DECF	33,1
	BTFSC	3,2
	RETURN
	MOVLW	10	;61 - visina 4100ft $1004
	MOVWF	34
	MOVLW	4
	MOVWF	35
	DECF	33,1
	BTFSC	3,2
	RETURN
	MOVLW	11	;62 - visina 4500ft $1194
	MOVWF	34
	MOVLW	94
	MOVWF	35
	DECF	33,1
	BTFSC	3,2
	RETURN
	MOVLW	13	;63 - visina 5000ft $1388
	MOVWF	34
	MOVLW	88
	MOVWF	35
	RETURN

;*** Nastavi konfiguracijo PICa 16F876/16F876A ***
	ORG	2007	;naslov konfiguracijske besede
	DW	3972	
;3: 11=/CP=off
;9: 1=/DEBUG=off, 00=WRT10=off, 1=/CPD=off 
;7: 0=LVP, 1=BODEN=on, 11=/CP=off
;2: 0=/PWRTE=on, 0=WDTE=off, 10=HSoscillator

;*** Nastavi zacetne vrednosti EEPROMa ***
	ORG	2103
	DW	40	;MIN jakost
	END

;*** Konec programa ***
